@@ -1,4 +1,3 @@
-import dlib #19.20.0
 import os
 import zipfile
 import bz2
@@ -10,6 +9,13 @@ class DlibResNet:
 	
 	def __init__(self):
 		
+		#this is not a must dependency
+		import dlib #19.20.0
+	
+		self.layers = [DlibMetaData()]
+		
+		#---------------------
+		
 		home = str(Path.home())
 		weight_file = home+'/.deepface/weights/dlib_face_recognition_resnet_model_v1.dat'
 		
@@ -59,4 +65,8 @@ class DlibResNet:
 		img_representation = np.array(img_representation)
 		img_representation = np.expand_dims(img_representation, axis = 0)
 		
-		return img_representation
\ No newline at end of file
+		return img_representation
+
+class DlibMetaData:
+	def __init__(self):
+		self.input_shape = [[1, 150, 150, 3]]
\ No newline at end of file
